-----------------------------
--sil00u_core.vhd
-----------------------------
library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.STD_LOGIC_ARITH.ALL;
use IEEE.STD_LOGIC_UNSIGNED.ALL;

entity sil00u_core is
  generic (
    C_CNT_WIDTH             : integer := 23
    );  
    Port ( 
                     SYS_CLK : in  STD_LOGIC;
           SYS_RST : in  STD_LOGIC;
           nSW : in  STD_LOGIC_VECTOR (0 to 2);
           nCODE : in  STD_LOGIC_VECTOR (0 to 3);
           seg_in1 : in  STD_LOGIC_VECTOR (0 to 3);
           seg_in2 : in  STD_LOGIC_VECTOR (0 to 3);
           seg_in3 : in  STD_LOGIC_VECTOR (0 to 3);
           le_trig : in  STD_LOGIC;
           SEG : out  STD_LOGIC_VECTOR (0 to 7);
           nSEL : out  STD_LOGIC_VECTOR (0 to 2);
           nLE : out  STD_LOGIC_VECTOR (0 to 3);
           sw : out  STD_LOGIC_VECTOR (0 to 2);
           code : out  STD_LOGIC_VECTOR (0 to 3);
           intr : out  STD_LOGIC
                     );
end sil00u_core;

architecture IMP of sil00u_core is
        signal count : STD_LOGIC_VECTOR(0 to C_CNT_WIDTH-1);
        signal le : STD_LOGIC_VECTOR(0 to 3);
        signal le_t : STD_LOGIC_VECTOR(0 to 3);
        signal seg_data : STD_LOGIC_VECTOR(0 to 3);

  component slot_counter
        generic (
            C_CNT_WIDTH             : integer := C_CNT_WIDTH
            );      
    port (
      SYS_CLK : in STD_LOGIC; 
      SYS_RST : in STD_LOGIC;   
            count : out STD_LOGIC_VECTOR(0 to C_CNT_WIDTH-1)
            );
    end component;

  component le_seq_blink
    port (
      SYS_CLK : in STD_LOGIC; 
      SYS_RST : in STD_LOGIC;           
      le_timing : in STD_LOGIC;
            le : out STD_LOGIC_VECTOR(0 to 3)
      );
  end component;

    component dynamic_ctrl 
    Port ( 
                     SYS_CLK : in  STD_LOGIC;
           SYS_RST : in  STD_LOGIC;
           nSEL : out  STD_LOGIC_VECTOR(0 to 2);
                     seg7_timing : in STD_LOGIC;
           seg_in1 : in  STD_LOGIC_VECTOR(0 to 3);
           seg_in2 : in  STD_LOGIC_VECTOR(0 to 3);
                     seg_in3 : in STD_LOGIC_VECTOR(0 to 3);
                     seg_data : out STD_LOGIC_VECTOR(0 to 3)
                     );
    end component;
    
    component seg7_decoder 
    Port ( 
           SEG : out  STD_LOGIC_VECTOR(0 to 7);
           seg_data : in  STD_LOGIC_VECTOR(0 to 3)
                     );
    end component;

begin

  slot_counter_0 : slot_counter   
    Port map(
      SYS_CLK => SYS_CLK,       
      SYS_RST => SYS_RST,
            count => count
      );

  le_seq_blink_0 : le_seq_blink   
    Port map(
      SYS_CLK => SYS_CLK,       
      SYS_RST => SYS_RST,
      le => le,
            le_timing => count(1)
      );

  dynamic_ctrl_0 : dynamic_ctrl
        Port map(
      SYS_CLK => SYS_CLK,       
      SYS_RST => SYS_RST,           
            nSEL=> nSEL,
            seg7_timing => count(8),
            seg_in1 => seg_in1,
            seg_in2 => seg_in2,
            seg_in3 => seg_in3,
            seg_data => seg_data
      );
            
    seg7_decoder_0 : seg7_decoder
        Port map(
            SEG => SEG,
            seg_data => seg_data
            );

    le_t <= le and "1111" when le_trig = '1' else "0000";   
    nLE <= not le_t;
    sw <= not nSW;
    code <= not nCODE;  
    intr <= count(4);

end IMP;

